﻿using System.Collections.Generic;
using System.Linq;

using Wintellect.PowerCollections;

namespace Library;

public class BookList
{
    private readonly Dictionary<string, OrderedBag<Book>> _booksByTitle;
    private readonly Dictionary<string, OrderedBag<Book>> _booksByAuthor;
    private readonly OrderedDictionary<decimal, Bag<Book>> _booksByPrice;
    private readonly Dictionary<string, Bag<Book>> _booksByTitleAndAuthor;

    public BookList()
    {
        this._booksByTitle = new ();
        this._booksByAuthor = new();
        this._booksByPrice = new();
        this._booksByTitleAndAuthor = new();
    }

    public void AddBook(string title, decimal price, string author)
    {
        Book book = new(title, price, author);

        this._booksByTitle.AppendValueToKey(title, book);
        this._booksByAuthor.AppendValueToKey(author, book);
        this._booksByPrice.AppendValueToKey(price, book);
        this._booksByTitleAndAuthor.AppendValueToKey(title + author, book);
    }

    public int DeleteByTitleAndAuthor(string title, string author)
    {
        string key = title + author;

        if (!this._booksByTitleAndAuthor.ContainsKey(key))
        {
            return 0;
        }

        Bag<Book> booksToBeDeleted = this._booksByTitleAndAuthor[key];

        this._booksByTitleAndAuthor.Remove(key);

        foreach (Book book in booksToBeDeleted)
        {
            this._booksByTitle[book.Title].Remove(book);
            this._booksByAuthor[book.Author].Remove(book);
            this._booksByPrice[book.Price].Remove(book);
        }

        return booksToBeDeleted.Count;
    }

    public IEnumerable<Book> FindBookByTitle(string title)
    {
        if (!this._booksByTitle.ContainsKey(title))
        {
            return Enumerable.Empty<Book>();
        }

        return this._booksByTitle[title];
    }

    public IEnumerable<Book> FindBookByAuthor(string author)
    {
        if (!this._booksByAuthor.ContainsKey(author))
        {
            return Enumerable.Empty<Book>();
        }

        return this._booksByAuthor[author];
    }

    public IEnumerable<Book> FindBooksByPriceRange(decimal from, decimal to)
    {
        List<Book> range = this._booksByPrice.Range(from, true, to, true)
            .Values
            .SelectMany(x => x)
            .ToList();

        range.Sort();

        return range;
    }
}
